# -*- coding: utf-8 -*-

import os
import sys

camplight_root = os.path.join(os.path.abspath(os.path.dirname(__file__)), '..')
sys.path.insert(0, camplight_root)


import pytest
from httpretty import HTTPretty
from camplight import Request, Campfire, Room, MessageType, Sound


def campfire_url(path=''):
    return 'https://foo.campfirenow.com' + path

def stub_get(path, *args, **kwargs):
    HTTPretty.register_uri(HTTPretty.GET, campfire_url(path), *args, **kwargs)

def stub_post(path, *args, **kwargs):
    HTTPretty.register_uri(HTTPretty.POST, campfire_url(path), *args, **kwargs)

def stub_put(path, *args, **kwargs):
    HTTPretty.register_uri(HTTPretty.PUT, campfire_url(path), *args, **kwargs)


class TestRoom(object):

    def setup_class(self):
        HTTPretty.enable()
        self.request = Request(campfire_url(), 'some_token')
        self.campfire = Campfire(self.request)
        self.room_id = 27121983
        self.room = Room(self.request, self.room_id)

    def teardown_class(self):
        HTTPretty.disable()

    def test_status(self):
        stub_get('/room/%s.json' % self.room_id, body="""
            {"room": {"name": "Danger", "topic": "No serious discussion"}}""")
        room = self.room.status()
        assert room['name'] == 'Danger'
        assert room['topic'] == 'No serious discussion'

    def test_recent(self):
        stub_get('/room/%s/recent.json' % self.room_id, body="""
            {"messages": [{"body": "Hello World", "type": "TextMessage"}]}""")
        messages = self.room.recent()
        assert len(messages) == 1
        assert messages[0]['body'] == 'Hello World'
        assert messages[0]['type'] == MessageType.TEXT

    def test_transcript(self):
        stub_get('/room/%s/transcript.json' % self.room_id, body="""
            {"messages": [{"body": "Hello World", "type": "TextMessage"}]}""")
        messages = self.room.transcript()
        assert len(messages) == 1
        assert messages[0]['body'] == 'Hello World'
        assert messages[0]['type'] == MessageType.TEXT

    def test_uploads(self):
        stub_get('/room/%s/uploads.json' % self.room_id, body="""
            {"uploads": [{"name": "file.png", "content_type": "image/png"}]}""")
        uploads = self.room.uploads()
        assert len(uploads) == 1
        assert uploads[0]['name'] == 'file.png'
        assert uploads[0]['content_type'] == 'image/png'

    def test_join(self):
        stub_post('/room/%s/join.json' % self.room_id, body='')
        assert self.room.join() == None

    def test_leave(self):
        stub_post('/room/%s/leave.json' % self.room_id, body='')
        assert self.room.leave() == None

    def test_lock(self):
        stub_post('/room/%s/lock.json' % self.room_id, body='')
        assert self.room.lock() == None

    def test_unlock(self):
        stub_post('/room/%s/unlock.json' % self.room_id, body='')
        assert self.room.unlock() == None

    def test_speak(self):
        body = b'{"message": {"body": "Hello World"}}'
        stub_post('/room/%s/speak.json' % self.room_id, body=body)
        message = self.room.speak('Hello World')
        assert message['body'] == 'Hello World'
        assert hasattr(message, 'type') == False
        assert HTTPretty.last_request.body == body

    def test_paste(self):
        body = b'{"message": {"body": "Hello World", "type": "PasteMessage"}}'
        stub_post('/room/%s/speak.json' % self.room_id, body=body)
        message = self.room.paste('Hello World')
        assert message['body'] == 'Hello World'
        assert message['type'] == MessageType.PASTE
        assert HTTPretty.last_request.body == body

    def test_play(self):
        body = b'{"message": {"body": "yeah", "type": "SoundMessage"}}'
        stub_post('/room/%s/speak.json' % self.room_id, body=body)
        message = self.room.play(Sound.YEAH)
        assert message['body'] == Sound.YEAH
        assert message['type'] == MessageType.SOUND
        assert HTTPretty.last_request.body == body

    def test_set_name(self):
        stub_put('/room/%s.json' % self.room_id, body='')
        assert self.room.set_name('Danger') == None
        assert HTTPretty.last_request.body == b'{"room": {"name": "Danger"}}'

    def test_set_topic(self):
        stub_put('/room/%s.json' % self.room_id, body='')
        assert self.room.set_topic('No serious discussion') == None
        assert HTTPretty.last_request.body == \
            b'{"room": {"topic": "No serious discussion"}}'


if __name__ == '__main__':
    pytest.main(__file__)



# -*- coding: utf-8 -*-

"""
camplight.api
~~~~~~~~~~~~~

This module implements the Campfire API.

"""

import requests
import json

from .exceptions import *

__all__ = ['Request', 'Campfire', 'Room', 'MessageType', 'Sound']


class Request(object):

    def __init__(self, url, token, verbose=None):
        self.url = url
        self._auth = (token, '')
        self.verbose = verbose

    def _request(self, method, path, data=None):
        url = self.url + path + '.json'

        headers = None
        if data is not None:
            data = json.dumps(data)
            headers = {'Content-Type': 'application/json'}

        r = requests.request(method, url, data=data, headers=headers,
                             auth=self._auth)
        r.raise_for_status()

        if self.verbose is not None:
            self.verbose.write(r.text + '\n')

        return r.json() if r.text.strip() else None

    def get(self, *args, **kwargs):
        return self._request('GET', *args, **kwargs)

    def post(self, *args, **kwargs):
        return self._request('POST', *args, **kwargs)

    def put(self, *args, **kwargs):
        return self._request('PUT', *args, **kwargs)


class Campfire(object):

    def __init__(self, request):
        self.request = request

    def account(self):
        return self.request.get('/account')['account']

    def rooms(self):
        return self.request.get('/rooms')['rooms']

    def _room_by_name(self, name):
        try:
            return [r for r in self.rooms() if r['name'] == name][0]
        except IndexError:
            raise RoomNotFoundError('Cannot find room "%s"' % name)

    def room(self, room_id):
        try:
            int(room_id)
        except (TypeError, ValueError, OverflowError):
            room_id = self._room_by_name(room_id)['id']
        return Room(self.request, room_id)

    def user(self, user_id=None):
        if user_id is None:
            user_id = 'me'
        return self.request.get('/users/%s' % user_id)['user']

    def presence(self):
        return self.request.get('/presence')['rooms']

    def search(self, term):
        return self.request.get('/search/%s' % term)['messages']


class Room(object):

    def __init__(self, request, room_id):
        self.request = request
        self.room_id = room_id
        self._path = '/room/%s' % self.room_id

    def status(self):
        return self.request.get(self._path)['room']

    def recent(self):
        return self.request.get(self._path + '/recent')['messages']

    def transcript(self):
        return self.request.get(self._path + '/transcript')['messages']

    def uploads(self):
        return self.request.get(self._path + '/uploads')['uploads']

    def join(self):
        self.request.post(self._path + '/join')

    def leave(self):
        self.request.post(self._path + '/leave')

    def lock(self):
        self.request.post(self._path + '/lock')

    def unlock(self):
        self.request.post(self._path + '/unlock')

    def speak(self, message, type_=None):
        params = {'body': message}
        if type_ is not None:
            params['type'] = type_
        data = {'message': params}
        return self.request.post(self._path + '/speak', data=data)['message']

    def paste(self, message):
        return self.speak(message, MessageType.PASTE)

    def play(self, sound):
        return self.speak(sound, MessageType.SOUND)

    def update(self, name=None, topic=None):
        params = {}
        if name is not None:
            params['name'] = name
        if topic is not None:
            params['topic'] = topic
        self.request.put(self._path, data={'room': params})

    def set_name(self, name):
        return self.update(name=name)

    def set_topic(self, topic):
        return self.update(topic=topic)


class MessageType(object):
    TEXT = 'TextMessage'
    PASTE = 'PasteMessage'
    SOUND = 'SoundMessage'
    TWEET = 'TweetMessage'


class Sound(object):
    # hard to keep this list up-to-date
    FIFTYSIXK = '56k'
    BUELLER = 'bueller'
    CRICKETS = 'crickets'
    DANGERZONE = 'dangerzone'
    DEEPER = 'deeper'
    DRAMA = 'drama'
    GREATJOB = 'greatjob'
    HORN = 'horn'
    HORROR = 'horror'
    INCONCEIVABLE = 'inconceivable'
    LIVE = 'live'
    LOGGINS = 'loggins'
    NOOOO = 'noooo'
    NYAN = 'nyan'
    OHMY = 'ohmy'
    OHYEAH = 'ohyeah'
    PUSHIT = 'pushit'
    RIMSHOT = 'rimshot'
    SAX = 'sax'
    SECRET = 'secret'
    TADA = 'tada'
    TMYK = 'tmyk'
    TROMBONE = 'trombone'
    VUVUZELA = 'vuvuzela'
    YEAH = 'yeah'
    YODEL = 'yodel'



# -*- coding: utf-8 -*-

"""
camplight.exceptions
~~~~~~~~~~~~~~~~~~~~

This module contains the exceptions raised by Camplight.

"""

from requests.exceptions import *


class CamplightException(RuntimeError):
    """Base class for Camplight exceptions."""


class RoomNotFoundError(CamplightException):
    """Campfire room does not exist."""



# -*- coding: utf-8 -*-

"""
camplight
~~~~~~~~~

Python implementation of Campfire API

"""

__author__ = 'Mathias Lafeldt <mathias.lafeldt@gmail.com>'
__license__ = 'MIT'
__version__ = '0.9.1'

from .api import *
from .exceptions import *



# -*- coding: utf-8 -*-

import os
import sys

camplight_root = os.path.join(os.path.abspath(os.path.dirname(__file__)), '..')
sys.path.insert(0, camplight_root)

import pytest
from httpretty import HTTPretty
from camplight import Request, Campfire, RoomNotFoundError, MessageType


def campfire_url(path=''):
    return 'https://foo.campfirenow.com' + path

def stub_get(path, *args, **kwargs):
    HTTPretty.register_uri(HTTPretty.GET, campfire_url(path), *args, **kwargs)


class TestCampfire(object):

    def setup_class(self):
        HTTPretty.enable()
        self.request = Request(campfire_url(), 'some_token')
        self.campfire = Campfire(self.request)

    def teardown_class(self):
        HTTPretty.disable()

    def test_account(self):
        stub_get('/account.json', body="""
            {"account": {"subdomain": "foo", "id": 12345678}}""")
        account = self.campfire.account()
        assert account['subdomain'] == 'foo'
        assert account['id'] == 12345678

    def test_rooms(self):
        stub_get('/rooms.json', body="""
            {"rooms": [{"name": "Serious"}, {"name": "Danger"}]}""")
        rooms = self.campfire.rooms()
        assert len(rooms) == 2
        assert rooms[0]['name'] == 'Serious'
        assert rooms[1]['name'] == 'Danger'

    def test_room_by_name(self):
        stub_get('/rooms.json', body="""
            {"rooms": [{"name": "Serious", "id": 1000},
                       {"name": "Danger",  "id": 2000}]}""")
        room = self.campfire.room('Serious')
        assert room.room_id == 1000

    def test_room_by_id(self):
        assert self.campfire.room(3000).room_id == 3000

    def test_room_not_found(self):
        stub_get('/rooms.json', body="""
            {"rooms": [{"name": "Serious", "id": 1000}]}""")
        with pytest.raises(RoomNotFoundError):
            self.campfire.room('Danger')

    def test_user_me(self):
        stub_get('/users/me.json', body="""
            {"user": {"name": "John Doe",
                      "email_address": "john.doe@gmail.com"}}""")
        user = self.campfire.user()
        assert user['name'] == 'John Doe'
        assert user['email_address'] == 'john.doe@gmail.com'

    def test_user_other(self):
        user_id = 6789
        stub_get('/users/%s.json' % user_id, body="""
            {"user": {"name": "Alan Turing"}}""")
        user = self.campfire.user(user_id)
        assert user['name'] == 'Alan Turing'

    def test_presence(self):
        stub_get('/presence.json', body="""{"rooms": [{"name": "Serious"}]}""")
        rooms = self.campfire.presence()
        assert len(rooms) == 1
        assert rooms[0]['name'] == 'Serious'

    def test_search(self):
        term = 'ohai'
        stub_get('/search/%s.json' % term, body="""
            {"messages": [{"body": "ohai world", "type": "TextMessage"}]}""")
        messages = self.campfire.search(term)
        assert len(messages) == 1
        assert messages[0]['body'] == 'ohai world'
        assert messages[0]['type'] == MessageType.TEXT


if __name__ == '__main__':
    pytest.main(__file__)



# -*- coding: utf-8 -*-

"""
camplight.cli
~~~~~~~~~~~~~

This module implements the command-line interface to the Campfire API.

"""

import sys
import os
import optparse

from .api import *
from .exceptions import *


def die(msg):
    sys.exit('error: %s' % msg)


def main(argv=None):
    usage = 'Usage: %prog [options] <command> [args]'
    parser = optparse.OptionParser(usage=usage)
    parser.add_option('-u', '--url',
                      help='set Campfire URL',
                      default=os.environ.get('CAMPFIRE_URL'))
    parser.add_option('-t', '--token',
                      help='set API token for authentication',
                      default=os.environ.get('CAMPFIRE_TOKEN'))
    parser.add_option('-r', '--room',
                      help='set Campfire room',
                      default=os.environ.get('CAMPFIRE_ROOM'))
    parser.add_option('-v', '--verbose',
                      help='be more verbose',
                      action='store_true',
                      default=os.environ.get('CAMPFIRE_VERBOSE'))
    opts, args = parser.parse_args(argv)

    if not opts.url:
        die('Campfire URL missing')
    if not opts.token:
        die('API token missing')
    if len(args) < 1:
        die('too few arguments')

    verbose = sys.stderr if opts.verbose else None
    request = Request(opts.url, opts.token, verbose)
    campfire = Campfire(request)

    cmd = args.pop(0)
    if cmd in ('account', 'rooms', 'user', 'presence', 'search'):
        func = getattr(campfire, cmd)
    elif cmd in ('status', 'recent', 'transcript', 'uploads',
                 'join', 'leave', 'lock', 'unlock', 'speak',
                 'paste', 'play', 'set-name', 'set-topic'):
        if opts.room is None:
            die('Campfire room missing')
        try:
            room = campfire.room(opts.room)
        except (RequestException, CamplightException) as e:
            die('%s: %s' % (e.__class__.__name__, e))
        func = getattr(room, cmd.replace('-', '_'))
    else:
        die('invalid command')

    try:
        data = func(*args)
    except TypeError:
        die('invalid arguments')
    except (RequestException, CamplightException) as e:
        die('%s: %s' % (e.__class__.__name__, e))

    if data:
        # HACK re-encode json for pretty output
        import json
        print(json.dumps(data, indent=4))



# -*- coding: utf-8 -*-

from setuptools import Command, find_packages, setup
import sys

install_requires = ['requests>=1.0.3']
tests_require = ['pytest', 'httpretty>=0.5.9']


class PyTest(Command):
    description = 'Runs the test suite.'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import pytest
        errno = pytest.main('test')
        sys.exit(errno)


setup(name='camplight',
      version='0.9.1',
      author='Mathias Lafeldt',
      author_email='mathias.lafeldt@gmail.com',
      url='https://github.com/mlafeldt/camplight',
      license='MIT',
      description='Python implementation of the Campfire API',
      long_description=open('README.md').read() + '\n\n' +
                       open('HISTORY.md').read(),
      classifiers=['Development Status :: 5 - Production/Stable',
                   'Intended Audience :: Developers',
                   'Natural Language :: English',
                   'License :: OSI Approved :: MIT License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2.6',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3.3'],
      packages=find_packages(),
      zip_safe=False,
      setup_requires=[],
      install_requires=install_requires,
      tests_require=tests_require,
      extras_require={'test': tests_require},
      entry_points="""
      # -*- Entry points: -*-
      [console_scripts]
      camplight=camplight.cli:main
      """,
      cmdclass={'test': PyTest})
